# Autogenerated by nbdev

d = { 'settings': { 'audience': 'Developers',
                'author': 'Clive',
                'author_email': 'clive@gm.net.nz',
                'branch': 'main',
                'copyright': 'Put your copyright info here',
                'custom_sidebar': 'False',
                'description': 'Common Module for Working with em6',
                'doc_baseurl': '/em6_api/',
                'doc_host': 'https://CJP123.github.io',
                'doc_path': '_docs',
                'git_url': 'https://github.com/CJP123/em6_api/',
                'host': 'github',
                'keywords': 'nbdev',
                'language': 'English',
                'lib_name': 'em6_api',
                'lib_path': 'em6_api',
                'license': 'apache2',
                'min_python': '3.9',
                'nbs_path': '.',
                'recursive': 'False',
                'repo': 'em6_api',
                'status': '2',
                'title': 'em6_api',
                'tst_flags': 'notest',
                'user': 'CJP123',
                'version': '0.0.1'},
  'syms': { 'em6_api.core': { 'em6_api.core.EM6': 'https://CJP123.github.io/em6_api/core.html#em6',
                              'em6_api.core.EM6.current_generation': 'https://CJP123.github.io/em6_api/core.html#em6.current_generation',
                              'em6_api.core.EM6.current_hvdc': 'https://CJP123.github.io/em6_api/core.html#em6.current_hvdc',
                              'em6_api.core.EM6.current_load': 'https://CJP123.github.io/em6_api/core.html#em6.current_load',
                              'em6_api.core.EM6.days_ago': 'https://CJP123.github.io/em6_api/core.html#em6.days_ago',
                              'em6_api.core.EM6.hvdc': 'https://CJP123.github.io/em6_api/core.html#em6.hvdc',
                              'em6_api.core.EM6.melted': 'https://CJP123.github.io/em6_api/core.html#em6.melted',
                              'em6_api.core.EM6.nodes': 'https://CJP123.github.io/em6_api/core.html#em6.nodes',
                              'em6_api.core.EM6.raw_scada': 'https://CJP123.github.io/em6_api/core.html#em6.raw_scada',
                              'em6_api.core.EM6.scada': 'https://CJP123.github.io/em6_api/core.html#em6.scada',
                              'em6_api.core.EM6.scada_generation': 'https://CJP123.github.io/em6_api/core.html#em6.scada_generation',
                              'em6_api.core.Emissions': 'https://CJP123.github.io/em6_api/core.html#emissions'}}}